@@ -1,6 +1,6 @@
 # -*- coding:utf-8 -*-
 
-# Copyright (c) 2009-2014 - Simon Conseil
+# Copyright (c) 2009-2016 - Simon Conseil
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to
@@ -95,8 +95,8 @@ def build(source, destination, debug, verbose, force, config, theme, title,
     corresponding values from the settings file.
 
     """
-    level = ((debug and logging.DEBUG) or (verbose and logging.INFO)
-             or logging.WARNING)
+    level = ((debug and logging.DEBUG) or (verbose and logging.INFO) or
+             logging.WARNING)
     init_logging(__name__, level=level)
     logger = logging.getLogger(__name__)
 
@@ -117,8 +117,17 @@ def build(source, destination, debug, verbose, force, config, theme, title,
         logger.error("Input directory not found: %s", settings['source'])
         sys.exit(1)
 
-    if not os.path.relpath(settings['destination'],
-                           settings['source']).startswith('..'):
+    # on windows os.path.relpath raises a ValueError if the two paths are on
+    # different drives, in that case we just ignore the exception as the two
+    # paths are anyway not relative
+    relative_check = True
+    try:
+        relative_check = os.path.relpath(settings['destination'],
+                                         settings['source']).startswith('..')
+    except ValueError:
+        pass
+
+    if not relative_check:
         logger.error("Output directory should be outside of the input "
                      "directory.")
         sys.exit(1)
@@ -139,9 +148,18 @@ def build(source, destination, debug, verbose, force, config, theme, title,
         logger.debug('Copy %s to %s', src, dst)
         copy(src, dst, symlink=settings['orig_link'])
 
-    print(('Done.\nProcessed {image} images ({image_skipped} skipped) and '
-           '{video} videos ({video_skipped} skipped) in {duration:.2f} '
-           'seconds.').format(duration=time.time() - start_time, **gal.stats))
+    stats = gal.stats
+
+    def format_stats(_type):
+        opt = ["{} {}".format(stats[_type + '_' + subtype], subtype)
+               for subtype in ('skipped', 'failed')
+               if stats[_type + '_' + subtype] > 0]
+        opt = ' ({})'.format(', '.join(opt)) if opt else ''
+        return '{} {}s{}'.format(stats[_type], _type, opt)
+
+    print('Done.\nProcessed {} and {} in {:.2f} seconds.'
+          .format(format_stats('image'), format_stats('video'),
+                  time.time() - start_time))
 
 
 def init_plugins(settings):
@@ -169,24 +187,76 @@ def init_plugins(settings):
 
 
 @main.command()
-@argument('path', default='_build')
+@argument('destination', default='_build')
 @option('-p', '--port', help="Port to use", default=8000)
-def serve(path, port):
+@option('-c', '--config', default=_DEFAULT_CONFIG_FILE,
+        show_default=True, help='Configuration file')
+def serve(destination, port, config):
     """Run a simple web server."""
-
-    if os.path.exists(path):
-        os.chdir(path)
-        Handler = server.SimpleHTTPRequestHandler
-        httpd = socketserver.TCPServer(("", port), Handler, False)
-        print(" * Running on http://127.0.0.1:{}/".format(port))
-
-        try:
-            httpd.allow_reuse_address = True
-            httpd.server_bind()
-            httpd.server_activate()
-            httpd.serve_forever()
-        except KeyboardInterrupt:
-            print('\nAll done!')
+    if os.path.exists(destination):
+        pass
+    elif os.path.exists(config):
+        settings = read_settings(config)
+        destination = settings.get('destination')
+        if not os.path.exists(destination):
+            sys.stderr.write("The '{}' directory doesn't exist, "
+                             "maybe try building first?"
+                             "\n".format(destination))
+            sys.exit(1)
     else:
-        sys.stderr.write("The '%s' directory doesn't exist.\n" % path)
+        sys.stderr.write("The {destination} directory doesn't exist "
+                         "and the config file ({config}) could not be "
+                         "read."
+                         "\n".format(destination=destination, config=config))
+        sys.exit(2)
+
+    print('DESTINATION : {}'.format(destination))
+    os.chdir(destination)
+    Handler = server.SimpleHTTPRequestHandler
+    httpd = socketserver.TCPServer(("", port), Handler, False)
+    print(" * Running on http://127.0.0.1:{}/".format(port))
+
+    try:
+        httpd.allow_reuse_address = True
+        httpd.server_bind()
+        httpd.server_activate()
+        httpd.serve_forever()
+    except KeyboardInterrupt:
+        print('\nAll done!')
+
+@main.command()
+@argument('target')
+@argument('keys', nargs=-1)
+@option('-o', '--overwrite', default=False, is_flag=True,
+        help='Overwrite existing .md file')
+def set_meta(target, keys, overwrite=False):
+    """Write metadata keys to .md file.
+
+    TARGET can be a media file or an album directory. KEYS are key/value pairs.
+
+    Ex, to set the title of test.jpg to "My test image":
+
+    sigal set_meta test.jpg title "My test image"
+    """
+
+    if not os.path.exists(target):
+        sys.stderr.write("The target {} does not exist.\n".format(target))
         sys.exit(1)
+    if len(keys) < 2 or len(keys) % 2 > 0:
+        sys.stderr.write("Need an even number of arguments.\n")
+        sys.exit(1)
+
+    if os.path.isdir(target):
+        descfile = os.path.join(target, 'index.md')
+    else:
+        descfile = os.path.splitext(target)[0] + '.md'
+    if os.path.exists(descfile) and not overwrite:
+        sys.stderr.write("Description file '{}' already exists. "
+                         "Use --overwrite to overwrite it.\n".format(descfile))
+        sys.exit(2)
+
+    with open(descfile, "w") as fp:
+        for i in range(len(keys)//2):
+            k,v = keys[i*2:(i+1)*2]
+            fp.write("{}: {}\n".format(k.capitalize(), v))
+    print("{} metadata key(s) written to {}".format(len(keys)//2, descfile))
