@@ -1,8 +1,9 @@
 # -*- coding:utf-8 -*-
 
-# Copyright (c) 2009-2014 - Simon Conseil
+# Copyright (c) 2009-2016 - Simon Conseil
 # Copyright (c) 2013      - Christophe-Marie Duquesne
 # Copyright (c) 2014      - Jonas Kaufmann
+# Copyright (c) 2015      - François D.
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to
@@ -28,7 +29,6 @@ import fnmatch
 import logging
 import multiprocessing
 import os
-import pickle
 import sys
 import zipfile
 
@@ -37,14 +37,14 @@ from collections import defaultdict
 from datetime import datetime
 from itertools import cycle
 from os.path import isfile, join, splitext
-from PIL import Image as PILImage
 
 from . import image, video, signals
-from .compat import UnicodeMixin, strxfrm, url_quote, text_type
-from .image import process_image, get_exif_tags, get_exif_data
+from .compat import PY2, UnicodeMixin, strxfrm, url_quote, text_type, pickle
+from .image import process_image, get_exif_tags, get_exif_data, get_size
 from .settings import get_thumb
 from .utils import (Devnull, copy, check_or_create_dir, url_from_path,
-                    read_markdown, cached_property)
+                    read_markdown, cached_property, is_valid_html5_video,
+                    get_mime)
 from .video import process_video
 from .writer import Writer
 
@@ -104,8 +104,6 @@ class Media(UnicodeMixin):
             copy(self.src_path, join(orig_path, self.src_filename),
                  symlink=s['orig_link'])
             return url_from_path(join(s['orig_dir'], self.src_filename))
-        else:
-            return None
 
     @property
     def thumbnail(self):
@@ -122,6 +120,7 @@ class Media(UnicodeMixin):
             try:
                 generator(self.src_path, self.thumb_path,
                           self.settings['thumb_size'],
+                          self.settings['thumb_video_delay'],
                           fit=self.settings['thumb_fit'])
             except Exception as e:
                 self.logger.error('Failed to generate thumbnail: %s', e)
@@ -145,7 +144,7 @@ class Image(Media):
     """Gather all informations on an image file."""
 
     type = 'image'
-    extensions = ('.jpg', '.jpeg', '.png')
+    extensions = ('.jpg', '.jpeg', '.png', '.gif')
 
     @cached_property
     def date(self):
@@ -161,10 +160,20 @@ class Image(Media):
         try:
             return (get_exif_data(self.src_path)
                     if self.ext in ('.jpg', '.jpeg') else None)
-        except (IOError, IndexError, TypeError, AttributeError):
+        except Exception:
             self.logger.warning(u'Could not read EXIF data from %s',
                                 self.src_path)
-            return None
+
+    @cached_property
+    def size(self):
+        return get_size(self.dst_path)
+
+    @cached_property
+    def thumb_size(self):
+        return get_size(self.thumb_path)
+
+    def has_location(self):
+        return self.exif is not None and 'gps' in self.exif
 
 
 class Video(Media):
@@ -175,11 +184,17 @@ class Video(Media):
 
     def __init__(self, filename, path, settings):
         super(Video, self).__init__(filename, path, settings)
-        base = splitext(filename)[0]
+        base, ext = splitext(filename)
         self.date = None
         self.src_filename = filename
-        self.filename = self.url = base + '.webm'
-        self.dst_path = join(settings['destination'], path, base + '.webm')
+        if not settings['use_orig'] or not is_valid_html5_video(ext):
+            video_format = settings['video_format']
+            ext = '.' + video_format
+            self.filename = self.url = base + ext
+            self.mime = get_mime(ext)
+            self.dst_path = join(settings['destination'], path, base + ext)
+        else:
+            self.mime = get_mime(ext)
 
 
 class Album(UnicodeMixin):
@@ -206,6 +221,7 @@ class Album(UnicodeMixin):
         self.name = path.split(os.path.sep)[-1]
         self.gallery = gallery
         self.settings = settings
+        self.subdirs = dirnames
         self.output_file = settings['output_filename']
         self._thumbnail = None
 
@@ -225,10 +241,6 @@ class Album(UnicodeMixin):
         self.index_url = url_from_path(os.path.relpath(
             settings['destination'], self.dst_path)) + '/' + self.url_ext
 
-        # sort sub-albums
-        dirnames.sort(key=strxfrm, reverse=settings['albums_sort_reverse'])
-        self.subdirs = dirnames
-
         #: List of all medias in the album (:class:`~sigal.gallery.Image` and
         #: :class:`~sigal.gallery.Video`).
         self.medias = medias = []
@@ -281,6 +293,11 @@ class Album(UnicodeMixin):
             for key, val in meta.items():
                 setattr(self, key, val)
 
+        try:
+            self.author = self.meta['author'][0]
+        except KeyError:
+            self.author = self.settings.get('author')
+
     def create_output_directories(self):
         """Create output directories for thumbnails and original images."""
         check_or_create_dir(self.dst_path)
@@ -293,10 +310,32 @@ class Album(UnicodeMixin):
             self.orig_path = join(self.dst_path, self.settings['orig_dir'])
             check_or_create_dir(self.orig_path)
 
+    def sort_subdirs(self, albums_sort_attr):
+        if self.subdirs:
+            if albums_sort_attr:
+                root_path = self.path if self.path != '.' else ''
+                if albums_sort_attr.startswith("meta."):
+                    meta_key = albums_sort_attr.split(".", 1)[1]
+                    key = lambda s: strxfrm(
+                        self.gallery.albums[join(root_path, s)].meta.get(meta_key, [''])[0])
+                else:
+                    key = lambda s: strxfrm(getattr(
+                    self.gallery.albums[join(root_path, s)], albums_sort_attr))
+            else:
+                key = strxfrm
+
+            self.subdirs.sort(key=key,
+                              reverse=self.settings['albums_sort_reverse'])
+
+        signals.albums_sorted.send(self)
+
     def sort_medias(self, medias_sort_attr):
         if self.medias:
             if medias_sort_attr == 'date':
                 key = lambda s: s.date or datetime.now()
+            elif medias_sort_attr.startswith('meta.'):
+                meta_key = medias_sort_attr.split(".", 1)[1]
+                key = lambda s: strxfrm(s.meta.get(meta_key, [''])[0])
             else:
                 key = lambda s: strxfrm(getattr(s, medias_sort_attr))
 
@@ -355,21 +394,29 @@ class Album(UnicodeMixin):
             for f in self.medias:
                 ext = splitext(f.filename)[1]
                 if ext.lower() in Image.extensions:
-                    try:
-                        im = PILImage.open(f.src_path)
-                    except:
-                        self.logger.error("Failed to open %s", f.src_path)
-                    else:
-                        if im.size[0] > im.size[1]:
-                            self._thumbnail = join(self.name, f.thumbnail)
-                            self.logger.debug(
-                                "Use 1st landscape image as thumbnail for %r :"
-                                " %s", self, self._thumbnail)
-                            return url_from_path(self._thumbnail)
+                    # Use f.size if available as it is quicker (in cache), but
+                    # fallback to the size of src_path if dst_path is missing
+                    size = f.size
+                    if size is None:
+                        size = get_size(f.src_path)
+
+                    if size['width'] > size['height']:
+                        self._thumbnail = join(self.name, f.thumbnail)
+                        self.logger.debug(
+                            "Use 1st landscape image as thumbnail for %r :"
+                            " %s", self, self._thumbnail)
+                        return url_from_path(self._thumbnail)
 
             # else simply return the 1st media file
             if not self._thumbnail and self.medias:
-                self._thumbnail = join(self.name, self.medias[0].thumbnail)
+                for media in self.medias:
+                    if media.thumbnail is not None:
+                        self._thumbnail = join(self.name, media.thumbnail)
+                        break
+                else:
+                    self.logger.warning("No thumbnail found for %r", self)
+                    return None
+
                 self.logger.debug("Use the 1st image as thumbnail for %r : %s",
                                   self, self._thumbnail)
                 return url_from_path(self._thumbnail)
@@ -410,6 +457,12 @@ class Album(UnicodeMixin):
         breadcrumb.reverse()
         return breadcrumb
 
+    @property
+    def show_map(self):
+        """Check if we have at least one photo with GPS location in the album
+        """
+        return any(image.has_location() for image in self.images)
+
     @property
     def zip(self):
         """Make a ZIP archive with all media files and return its path.
@@ -422,7 +475,7 @@ class Album(UnicodeMixin):
 
         if zip_gallery and len(self) > 0:
             archive_path = join(self.dst_path, zip_gallery)
-            archive = zipfile.ZipFile(archive_path, 'w')
+            archive = zipfile.ZipFile(archive_path, 'w', allowZip64=True)
             attr = ('src_path' if self.settings['zip_media_format'] == 'orig'
                     else 'dst_path')
 
@@ -436,8 +489,6 @@ class Album(UnicodeMixin):
             archive.close()
             self.logger.debug('Created ZIP archive %s', archive_path)
             return zip_gallery
-        else:
-            return None
 
 
 class Gallery(object):
@@ -500,6 +551,11 @@ class Gallery(object):
                 album.create_output_directories()
                 albums[relpath] = album
 
+        with progressbar(albums.values(), label="Sorting albums",
+                         file=self.progressbar_target) as progress_albums:
+            for album in progress_albums:
+                album.sort_subdirs(settings['albums_sort_attr'])
+
         with progressbar(albums.values(), label="Sorting media",
                          file=self.progressbar_target) as progress_albums:
             for album in progress_albums:
@@ -554,7 +610,10 @@ class Gallery(object):
             # 63 is the total length of progressbar, label, percentage, etc
             available_length = get_terminal_size()[0] - 64
             if x and available_length > 10:
-                return text_type(x.name)[:available_length].encode('utf-8')
+                text = text_type(x.name)[:available_length]
+                if PY2:
+                    text = text.encode('utf-8')
+                return text
             else:
                 return ""
 
@@ -570,9 +629,9 @@ class Gallery(object):
         bar_opt = {'label': "Processing files",
                    'show_pos': True,
                    'file': self.progressbar_target}
+        failed_files = []
 
         if self.pool:
-            failed_files = []
             try:
                 with progressbar(length=len(media_list), **bar_opt) as bar:
                     for res in self.pool.imap_unordered(worker, media_list):
@@ -591,14 +650,16 @@ class Gallery(object):
                     "defined in the settings file, which can't be serialized.",
                     exc_info=True)
                 sys.exit('Abort')
-
-            if failed_files:
-                self.remove_files(failed_files)
-            print('')
         else:
             with progressbar(media_list, **bar_opt) as medias:
                 for media_item in medias:
-                    process_file(media_item)
+                    res = process_file(media_item)
+                    if res:
+                        failed_files.append(res)
+
+        if failed_files:
+            self.remove_files(failed_files)
+        print('')
 
         if self.settings['write_html']:
             writer = Writer(self.settings, index_title=self.title)
