@@ -1,6 +1,7 @@
 # -*- coding:utf-8 -*-
 
-# Copyright (c) 2009-2014 - Simon Conseil
+# Copyright (c) 2009-2016 - Simon Conseil
+# Copyright (c) 2015 - François D.
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to
@@ -31,8 +32,11 @@
 
 import logging
 import os
+import PIL
 import pilkit.processors
 import sys
+import warnings
+import fractions
 
 from copy import deepcopy
 from datetime import datetime
@@ -42,8 +46,8 @@ from PIL import ImageOps
 from pilkit.processors import Transpose
 from pilkit.utils import save_image
 
-from . import compat, signals
-from .settings import get_thumb
+from . import compat, signals, utils
+from .settings import get_thumb, Status
 
 
 def _has_exif_tags(img):
@@ -61,6 +65,11 @@ def generate_image(source, outname, settings, options=None):
     """
 
     logger = logging.getLogger(__name__)
+
+    if settings['use_orig'] or source.endswith('.gif'):
+        utils.copy(source, outname, symlink=settings['orig_link'])
+        return
+
     img = PILImage.open(source)
     original_format = img.format
 
@@ -107,7 +116,7 @@ def generate_image(source, outname, settings, options=None):
     save_image(img, outname, outformat, options=options, autoconvert=True)
 
 
-def generate_thumbnail(source, outname, box, fit=True, options=None):
+def generate_thumbnail(source, outname, box, delay, fit=True, options=None):
     """Create a thumbnail image."""
 
     logger = logging.getLogger(__name__)
@@ -128,6 +137,7 @@ def process_image(filepath, outpath, settings):
     """Process one image: resize, create thumbnail."""
 
     logger = logging.getLogger(__name__)
+    logger.info('Processing %s', filepath)
     filename = os.path.split(filepath)[1]
     outname = os.path.join(outpath, filename)
     ext = os.path.splitext(filename)[1]
@@ -141,26 +151,63 @@ def process_image(filepath, outpath, settings):
 
     try:
         generate_image(filepath, outname, settings, options=options)
+
+        if settings['make_thumbs']:
+            thumb_name = os.path.join(outpath, get_thumb(settings, filename))
+            generate_thumbnail(outname, thumb_name, settings['thumb_size'],
+                               settings['thumb_video_delay'],
+                               fit=settings['thumb_fit'], options=options)
     except Exception as e:
-        logger.error('Failed to process image: %s', e)
-        return
+        logger.info('Failed to process: %r', e)
+        if logger.getEffectiveLevel() == logging.DEBUG:
+            raise
+        else:
+            return Status.FAILURE
 
-    if settings['make_thumbs']:
-        thumb_name = os.path.join(outpath, get_thumb(settings, filename))
-        generate_thumbnail(outname, thumb_name, settings['thumb_size'],
-                           fit=settings['thumb_fit'], options=options)
+    return Status.SUCCESS
 
 
-def _get_exif_data(filename):
-    """Return a dict with EXIF data."""
+def get_size(file_path):
+    """Return image size (width and height)."""
+    try:
+        im = PILImage.open(file_path)
+    except (IOError, IndexError, TypeError, AttributeError) as e:
+        logger = logging.getLogger(__name__)
+        logger.error("Could not read size of %s due to %r", file_path, e)
+    else:
+        width, height = im.size
+        return {
+            'width': width,
+            'height': height
+        }
+
+
+def get_exif_data(filename):
+    """Return a dict with the raw EXIF data."""
+
+    logger = logging.getLogger(__name__)
+
+    if PIL.PILLOW_VERSION == '3.0.0':
+        warnings.warn('Pillow 3.0.0 is broken with EXIF data, consider using '
+                      'another version if you want to use EXIF data.')
 
     img = PILImage.open(filename)
-    exif = img._getexif() or {}
+    try:
+        exif = img._getexif() or {}
+    except ZeroDivisionError:
+        logger.warning('Failed to read EXIF data.')
+        return None
+
     data = {TAGS.get(tag, tag): value for tag, value in exif.items()}
 
     if 'GPSInfo' in data:
-        data['GPSInfo'] = {GPSTAGS.get(tag, tag): value
-                           for tag, value in data['GPSInfo'].items()}
+        try:
+            data['GPSInfo'] = {GPSTAGS.get(tag, tag): value
+                               for tag, value in data['GPSInfo'].items()}
+        except AttributeError:
+            logger = logging.getLogger(__name__)
+            logger.info('Failed to get GPS Info')
+            del data['GPSInfo']
     return data
 
 
@@ -173,60 +220,77 @@ def dms_to_degrees(v):
     return d + (m / 60.0) + (s / 3600.0)
 
 
-def get_exif_tags(source):
-    """Read EXIF tags from file @source and return a tuple of two dictionaries,
-    the first one containing the raw EXIF data, the second one a simplified
-    version with common tags.
-    """
+def get_exif_tags(data):
+    """Make a simplified version with common tags from raw EXIF data."""
 
     logger = logging.getLogger(__name__)
-
-    if os.path.splitext(source)[1].lower() not in ('.jpg', '.jpeg'):
-        return (None, None)
-
-    try:
-        data = _get_exif_data(source)
-    except (IOError, IndexError, TypeError, AttributeError):
-        logger.warning(u'Could not read EXIF data from %s', source)
-        return (None, None)
-
     simple = {}
 
-    # Provide more accessible tags in the 'simple' key
+    for tag in ('Model', 'Make', 'LensModel'):
+        if tag in data:
+            if isinstance(data[tag], tuple):
+                simple[tag] = data[tag][0].strip()
+            else:
+                simple[tag] = data[tag].strip()
+
     if 'FNumber' in data:
         fnumber = data['FNumber']
-        simple['fstop'] = float(fnumber[0]) / fnumber[1]
+        try:
+            simple['fstop'] = float(fnumber[0]) / fnumber[1]
+        except IndexError:
+            # Pillow == 3.0
+            simple['fstop'] = float(fnumber[0])
+        except Exception:
+            logger.debug('Skipped invalid FNumber: %r', fnumber, exc_info=True)
 
     if 'FocalLength' in data:
         focal = data['FocalLength']
-        simple['focal'] = round(float(focal[0]) / focal[1])
+        try:
+            simple['focal'] = round(float(focal[0]) / focal[1])
+        except IndexError:
+            # Pillow == 3.0
+            simple['focal'] = round(focal[0])
+        except Exception:
+            logger.debug('Skipped invalid FocalLength: %r', focal,
+                         exc_info=True)
 
     if 'ExposureTime' in data:
-        if isinstance(data['ExposureTime'], tuple):
-            simple['exposure'] = '{0}/{1}'.format(*data['ExposureTime'])
-        elif isinstance(data['ExposureTime'], int):
-            simple['exposure'] = str(data['ExposureTime'])
+        exptime = data['ExposureTime']
+        if isinstance(exptime, tuple):
+            try:
+                simple['exposure'] = str(fractions.Fraction(exptime[0],
+                                                            exptime[1]))
+            except IndexError:
+                # Pillow == 3.0
+                simple['exposure'] = exptime[0]
+            except ZeroDivisionError:
+                logger.info('Invalid ExposureTime: %r', exptime)
+        elif isinstance(exptime, int):
+            simple['exposure'] = str(exptime)
         else:
-            logger.warning('Unknown format for ExposureTime: %r (%s)',
-                           data['ExposureTime'], source)
+            logger.info('Unknown format for ExposureTime: %r', exptime)
 
-    if 'ISOSpeedRatings' in data:
-        simple['iso'] = data['ISOSpeedRatings']
+    if data.get('ISOSpeedRatings'):
+        if isinstance(data['ISOSpeedRatings'], tuple):
+            # Pillow == 3.0
+            simple['iso'] = data['ISOSpeedRatings'][0]
+        else:
+            simple['iso'] = data['ISOSpeedRatings']
 
     if 'DateTimeOriginal' in data:
         try:
             # Remove null bytes at the end if necessary
             date = data['DateTimeOriginal'].rsplit('\x00')[0]
+        except AttributeError:
+            # Pillow == 3.0
+            date = data['DateTimeOriginal'][0]
+
+        try:
             simple['dateobj'] = datetime.strptime(date, '%Y:%m:%d %H:%M:%S')
             dt = simple['dateobj'].strftime('%A, %d. %B %Y')
-
-            if compat.PY2:
-                simple['datetime'] = dt.decode('utf8')
-            else:
-                simple['datetime'] = dt
+            simple['datetime'] = dt.decode('utf8') if compat.PY2 else dt
         except (ValueError, TypeError) as e:
-            logger.warning(u'Could not parse DateTimeOriginal of %s: %s',
-                           source, e)
+            logger.info(u'Could not parse DateTimeOriginal: %s', e)
 
     if 'GPSInfo' in data:
         info = data['GPSInfo']
@@ -240,13 +304,11 @@ def get_exif_tags(source):
                 lat = dms_to_degrees(lat_info)
                 lon = dms_to_degrees(lon_info)
             except (ZeroDivisionError, ValueError):
-                logger.warning('Failed to read GPS info for %s', source)
-                lat = lon = None
-
-            if lat and lon:
+                logger.info('Failed to read GPS info')
+            else:
                 simple['gps'] = {
                     'lat': - lat if lat_ref_info != 'N' else lat,
                     'lon': - lon if lon_ref_info != 'E' else lon,
                 }
 
-    return (data, simple)
+    return simple
