@@ -1,6 +1,6 @@
 # -*- coding:utf-8 -*-
 
-# Copyright (c) 2009-2014 - Simon Conseil
+# Copyright (c) 2009-2016 - Simon Conseil
 # Copyright (c) 2013      - Christophe-Marie Duquesne
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -27,17 +27,15 @@ import os
 from os.path import abspath, isabs, join, normpath
 from pprint import pformat
 
-from .compat import PY2
+from .compat import PY2, text_type
 
 
 _DEFAULT_CONFIG = {
-    'adjust_options': {'color': 1.0, 'brightness': 1.0,
-                       'contrast': 1.0, 'sharpness': 1.0},
+    'albums_sort_attr': 'name',
     'albums_sort_reverse': False,
     'autorotate_images': True,
     'colorbox_column_size': 4,
     'copy_exif_data': False,
-    'copyright': '',
     'destination': '_build',
     'files_to_copy': (),
     'google_analytics': '',
@@ -50,11 +48,17 @@ _DEFAULT_CONFIG = {
     'keep_orig': False,
     'links': '',
     'locale': '',
+    'make_thumbs': True,
     'medias_sort_attr': 'filename',
     'medias_sort_reverse': False,
-    'make_thumbs': True,
+    'mp4_options': ['-crf', '23', '-strict', '-2'],
     'orig_dir': 'original',
     'orig_link': False,
+    'output_filename': 'index.html',
+    'piwik': {'tracker_url': '', 'site_id': 0},
+    'plugin_paths': [],
+    'plugins': [],
+    'show_map': False,
     'source': '',
     'theme': 'colorbox',
     'thumb_dir': 'thumbnails',
@@ -62,7 +66,13 @@ _DEFAULT_CONFIG = {
     'thumb_prefix': '',
     'thumb_size': (200, 150),
     'thumb_suffix': '',
+    'thumb_video_delay': '0',
+    'title': '',
+    'use_assets_cdn': True,
+    'use_orig': False,
+    'video_format': 'webm',
     'video_size': (480, 360),
+    'watermark': '',
     'webm_options': ['-crf', '10', '-b:v', '1.6M',
                      '-qmin', '4', '-qmax', '63'],
     'write_html': True,
@@ -71,6 +81,11 @@ _DEFAULT_CONFIG = {
 }
 
 
+class Status(object):
+    SUCCESS = 0
+    FAILURE = 1
+
+
 def get_thumb(settings, filename):
     """Return the path to the thumb.
 
@@ -90,7 +105,7 @@ def get_thumb(settings, filename):
     name, ext = os.path.splitext(filen)
 
     # FIXME: replace this list with Video.extensions
-    if ext in ('.MOV', '.mov', '.avi', '.mp4', '.webm', '.ogv'):
+    if ext.lower() in ('.mov', '.avi', '.mp4', '.webm', '.ogv'):
         ext = '.jpg'
     return join(path, settings['thumb_dir'], settings['thumb_prefix'] +
                 name + settings['thumb_suffix'] + ext)
@@ -102,10 +117,10 @@ def read_settings(filename=None):
     logger = logging.getLogger(__name__)
     logger.info("Reading settings ...")
     settings = _DEFAULT_CONFIG.copy()
-    settings_path = os.path.dirname(filename)
 
     if filename:
         logger.debug("Settings file: %s", filename)
+        settings_path = os.path.dirname(filename)
         tempdict = {}
 
         with open(filename) as f:
@@ -116,23 +131,27 @@ def read_settings(filename=None):
                         if k not in ['__builtins__'])
 
         # Make the paths relative to the settings file
-        paths = ['source', 'destination']
+        paths = ['source', 'destination', 'watermark']
 
-        if os.path.isdir(join(settings_path, settings['theme'])):
+        if os.path.isdir(join(settings_path, settings['theme'])) and \
+                os.path.isdir(join(settings_path, settings['theme'],
+                                   'templates')):
             paths.append('theme')
 
         enc = locale.getpreferredencoding() if PY2 else None
 
         for p in paths:
+            # paths must to be unicode strings so that os.walk will return
+            # unicode dirnames and filenames
+            if PY2 and isinstance(settings[p], str):
+                settings[p] = settings[p].decode(enc)
             path = settings[p]
             if path and not isabs(path):
                 settings[p] = abspath(normpath(join(settings_path, path)))
                 logger.debug("Rewrite %s : %s -> %s", p, path, settings[p])
 
-            # paths must to be unicode strings so that os.walk will return
-            # unicode dirnames and filenames
-            if PY2 and isinstance(settings[p], str):
-                settings[p] = settings[p].decode(enc)
+        if settings['title'] and not isinstance(settings['title'], text_type):
+            settings['title'] = settings['title'].decode('utf8')
 
     for key in ('img_size', 'thumb_size', 'video_size'):
         w, h = settings[key]
